package main

import "fmt"

// Define the Person struct
type Person struct {
    Name string
    Age  int
}

// Method for Person
func (p Person) Greet() string {
    return "Hello, my name is " + p.Name
}

// Define the Employee struct, embedding Person
type Employee struct {
    Person     // Embedded struct
    Position   string
    Department string
}

// Additional method for Employee
func (e Employee) DescribeRole() string {
    return e.Position + " in " + e.Department
}

func main() {
    // Create an Employee instance
    emp := Employee{
        Person:    Person{Name: "Alice", Age: 30},
        Position:  "Software Engineer",
        Department: "Engineering",
    }
    
    // Go’s embedding feature, promotes all the embedded struct’s 
    // fields and methods to the outer struct

    // Access Person's Greet method through Employee
    fmt.Println(emp.Greet())          // Output: Hello, my name is Alice
                                      // Greet is a method on Person
                                      // here an Employee is calling it

    // Access Employee's DescribeRole method
    fmt.Println(emp.DescribeRole())    // Output: Software Engineer in Engineering
                                       // DescribeRole is a method in Employee
}
